@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2021 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -84,15 +84,19 @@
 #  define XFF_PARSE 1
 #endif
 
-/* avcodec_decode_video() */
-#if LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(52,32,0)
+/* avcodec_decode_video* (), avcodec_send_packet (), avcodec_receive_frame () */
+#if LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(57,106,102)
+#  define XFF_VIDEO 3
+#elif LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(52,32,0)
 #  define XFF_VIDEO 2
 #else
 #  define XFF_VIDEO 1
 #endif
 
-/* avcodec_decode_audio() */
-#if LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(54,0,0)
+/* avcodec_decode_audio* (), avcodec_send_packet (), avcodec_receive_frame () */
+#if LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(57,106,102)
+#  define XFF_AUDIO 5
+#elif LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(54,0,0)
 #  define XFF_AUDIO 4
 #elif LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(52,32,0)
 #  define XFF_AUDIO 3
@@ -100,8 +104,10 @@
 #  define XFF_AUDIO 2
 #endif
 
-/* avcodec_encode_video(), av_packet_unref */
-#if LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(55,25,100)
+/* avcodec_encode_video(), av_packet_unref, avcodec_send_frame (), avcodec_receive_packet () */
+#if LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(57,106,102)
+#  define XFF_ENCVIDEO 3
+#elif LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(55,25,100)
 #  define XFF_ENCVIDEO 2
 #else
 #  define XFF_ENCVIDEO 1
@@ -124,6 +130,12 @@
 #  define XFF_PALETTE 3
 #endif
 
+#if LIBAVCODEC_VERSION_INT < XFF_INT_VERSION(59,0,100) /** << revise this */
+#  define XFF_VAAPI 1 /** << libavcodec/vaapi.h */
+#else
+#  define XFF_VAAPI 2 /** << libavutil/hwcontext.h, libavutil/hwcontext_vaapi.h */
+#endif
+
 #if LIBAVUTIL_VERSION_INT >= XFF_INT_VERSION(52,0,0)
 #  define PIX_FMT_NONE      AV_PIX_FMT_NONE
 #  define PIX_FMT_YUV420P   AV_PIX_FMT_YUV420P
@@ -132,7 +144,6 @@
 #  define PIX_FMT_YUVJ444P  AV_PIX_FMT_YUVJ444P
 #  define PIX_FMT_YUV410P   AV_PIX_FMT_YUV410P
 #  define PIX_FMT_YUV411P   AV_PIX_FMT_YUV411P
-#  define PIX_FMT_VAAPI_VLD AV_PIX_FMT_VAAPI_VLD
 #  define PIX_FMT_ARGB      AV_PIX_FMT_ARGB
 #  define PIX_FMT_BGRA      AV_PIX_FMT_BGRA
 #  define PIX_FMT_RGB24     AV_PIX_FMT_RGB24
@@ -144,8 +155,17 @@
 #  define PIX_FMT_PAL8      AV_PIX_FMT_PAL8
 #  define PixelFormat       AVPixelFormat
 /* video_out/video_out_vaapi */
-#  define PIX_FMT_VAAPI_IDCT AV_PIX_FMT_VAAPI_IDCT
-#  define PIX_FMT_VAAPI_MOCO AV_PIX_FMT_VAAPI_MOCO
+#  if LIBAVCODEC_VERSION_INT < XFF_INT_VERSION(59,0,100) /** << revise this */
+#    define PIX_FMT_VAAPI_VLD AV_PIX_FMT_VAAPI_VLD
+#    define PIX_FMT_VAAPI_IDCT AV_PIX_FMT_VAAPI_IDCT
+#    define PIX_FMT_VAAPI_MOCO AV_PIX_FMT_VAAPI_MOCO
+#  else
+#    define PIX_FMT_VAAPI_VLD AV_PIX_FMT_VAAPI
+#    define PIX_FMT_VAAPI_IDCT AV_PIX_FMT_VAAPI
+#    define PIX_FMT_VAAPI_MOCO AV_PIX_FMT_VAAPI
+#  endif
+
+#  define CODEC_FLAG_BITEXACT AV_CODEC_FLAG_BITEXACT
 #endif
 
 #if LIBAVCODEC_VERSION_INT >= XFF_INT_VERSION(54,25,0)
@@ -182,6 +202,18 @@
 #  define XFF_AV_BUFFER 1
 #endif
 
+/* 0 (no), 1 (yes), 2 (deprecated but still needed to make direct rendering work) */
+#if LIBAVCODEC_VERSION_INT < XFF_INT_VERSION(55,0,100)
+#  define XFF_THREAD_SAFE_CB 0
+#elif LIBAVCODEC_VERSION_INT < XFF_INT_VERSION(59,0,100)
+#  define XFF_THREAD_SAFE_CB 1
+#elif LIBAVCODEC_VERSION_INT < XFF_INT_VERSION(60,0,0)
+#  define XFF_THREAD_SAFE_CB 2
+#else
+/* now callbacks shall always be thread safe. */
+#  define XFF_THREAD_SAFE_CB 0
+#endif
+
 /* function aliases */
 
 #if LIBAVCODEC_VERSION_INT < XFF_INT_VERSION(52,66,0)
@@ -229,9 +261,17 @@
 #endif
 
 #if LIBAVCODEC_VERSION_INT < XFF_INT_VERSION(57,12,100)
-#define XFF_PACKET_UNREF av_free_packet
+#  define XFF_PACKET_DECL(_p) AVPacket _p##_stat, *_p
+#  define XFF_PACKET_NEW(_p) _p = &_p##_stat, av_init_packet (_p)
+#  define XFF_PACKET_UNREF(_p) av_free_packet (_p)
+#elif LIBAVCODEC_VERSION_INT < XFF_INT_VERSION(59,0,100) /** << revise this */
+#  define XFF_PACKET_DECL(_p) AVPacket _p##_stat, *_p
+#  define XFF_PACKET_NEW(_p) _p = &_p##_stat, av_init_packet (_p)
+#  define XFF_PACKET_UNREF(_p) av_packet_unref (_p)
 #else
-#define XFF_PACKET_UNREF av_packet_unref
+#  define XFF_PACKET_DECL(_p) AVPacket *_p
+#  define XFF_PACKET_NEW(_p) _p = av_packet_alloc ()
+#  define XFF_PACKET_UNREF(_p) av_packet_free (&(_p))
 #endif
 
 #ifndef AV_INPUT_BUFFER_PADDING_SIZE
@@ -244,5 +284,11 @@
 #  define AV_CODEC_FLAG2_FAST CODEC_FLAG2_FAST
 #endif
 
+#if LIBAVCODEC_VERSION_INT < XFF_INT_VERSION(58,10,100)
+#  define XFF_AVCODEC_REGISTER_ALL() avcodec_register_all()
+#else
+#  define XFF_AVCODEC_REGISTER_ALL() do {} while(0)
+#endif
 
 #endif /* XINE_AVCODEC_COMPAT_H */
+
