@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2016 the xine project
+ * Copyright (C) 2004-2021 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -133,14 +133,8 @@ typedef struct {
 
   off_t                data_start;
   off_t                data_size;
-
-  int                  seek_flag;  /* this is set when a seek just occurred */
 } demux_iff_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_iff_class_t;
-
 
 /* Decode delta encoded data from n byte source
  * buffer into double as long dest buffer, given initial data
@@ -173,7 +167,7 @@ static int read_iff_chunk(demux_iff_t *this) {
   unsigned char signature[IFF_SIGNATURE_SIZE];
   unsigned char buffer[512];
   unsigned int  keep_on_reading = 1;
-  uint32_t      junk_size;
+  uint32_t      junk_size, junk_type;
 
   while ( keep_on_reading == 1 ) {
     if (this->input->read(this->input, signature, IFF_JUNK_SIZE) == IFF_JUNK_SIZE) {
@@ -188,8 +182,9 @@ static int read_iff_chunk(demux_iff_t *this) {
         if (this->input->read(this->input, &signature[7], 1) != 1)
           return 0;
       }
+      junk_type = _X_BE_32(&signature[0]);
       junk_size = _X_BE_32(&signature[4]);
-      switch( _X_BE_32(&signature[0]) ) {
+      switch (junk_type) {
         case IFF_CMAP_CHUNK:
         case IFF_BODY_CHUNK:
         case IFF_DLTA_CHUNK:
@@ -207,7 +202,7 @@ static int read_iff_chunk(demux_iff_t *this) {
           break;
       }
 
-      switch( _X_BE_32(&signature[0]) ) {
+      switch (junk_type) {
         case IFF_FORM_CHUNK:
           if (this->input->read(this->input, buffer, 4) != 4)
             return 0;
@@ -301,6 +296,8 @@ static int read_iff_chunk(demux_iff_t *this) {
           /* not yet implemented */
           break;
         case IFF_CHAN_CHUNK:
+          if (!this->vhdr)
+            break;
           this->chan_settings           = _X_BE_32(&buffer[0]);
           switch( this->chan_settings ) {
             case STEREO:
@@ -325,6 +322,8 @@ static int read_iff_chunk(demux_iff_t *this) {
           }
           break;
         case IFF_PAN_CHUNK:
+          if (!this->vhdr)
+            break;
           this->chan_settings           = PAN;
           this->pan_sposition           = _X_BE_32(&buffer[0]);
           this->audio_channels          = 2;
@@ -525,6 +524,8 @@ static int read_iff_chunk(demux_iff_t *this) {
           break;
         case IFF_BODY_CHUNK:
         case IFF_DLTA_CHUNK:
+          if (!this->vhdr)
+            break;
           this->data_start              = this->input->get_current_pos(this->input);
           this->data_size               = junk_size;
           switch( this->iff_type )
@@ -543,6 +544,7 @@ static int read_iff_chunk(demux_iff_t *this) {
               this->audio_bytes_per_second         = this->audio_channels *
                                                      (this->audio_bits / 8) *
                                                      this->vhdr->samplesPerSec;
+              if (this->audio_channels > 0 && this->vhdr->samplesPerSec > 0)
               this->running_time        = ((this->vhdr->oneShotHiSamples +
                                             this->vhdr->repeatHiSamples) *
                                            1000 / this->vhdr->samplesPerSec) /
@@ -606,7 +608,6 @@ static int open_iff_file(demux_iff_t *this) {
   this->running_time                    = 0;
   this->data_start                      = 0;
   this->data_size                       = 0;
-  this->seek_flag                       = 0;
   this->audio_interleave_buffer         = 0;
   this->audio_interleave_buffer_size    = 0;
   this->audio_read_buffer               = 0;
@@ -672,7 +673,9 @@ static int open_iff_file(demux_iff_t *this) {
     return 0;
 
   /* file is qualified; skip over the header bytes in the stream */
-  this->input->seek(this->input, IFF_SIGNATURE_SIZE, SEEK_SET);
+  if (this->input->seek(this->input, IFF_SIGNATURE_SIZE, SEEK_SET)
+      != IFF_SIGNATURE_SIZE)
+    return 0;
 
   return read_iff_chunk( this );
 
@@ -1092,10 +1095,11 @@ static int demux_iff_seek (demux_plugin_t *this_gen,
   start_pos = (off_t) ( (double) start_pos / 65535 *
               this->data_size );
 
+  (void)start_time;
+
   switch( this->iff_type ) {
     case IFF_8SVX_CHUNK:
     case IFF_16SV_CHUNK:
-      this->seek_flag                   = 1;
       this->status                      = DEMUX_OK;
       _x_demux_flush_engine (this->stream);
 
@@ -1108,10 +1112,10 @@ static int demux_iff_seek (demux_plugin_t *this_gen,
       this->audio_position              = (start_pos < 0) ? 0 :
                                           ((start_pos >= this->data_size) ?
                                            this->data_size : start_pos);
+      break;
     case IFF_ILBM_CHUNK:
     case IFF_ANIM_CHUNK:
       /* disable seeking for ILBM and ANIM */
-      this->seek_flag                   = 0;
       if( !playing ) {
         this->status = DEMUX_OK;
       }
@@ -1167,11 +1171,15 @@ static int demux_iff_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_iff_get_capabilities(demux_plugin_t *this_gen){
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_iff_get_optional_data(demux_plugin_t *this_gen,
 					void *data, int data_type){
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -1180,7 +1188,10 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_iff_t   *this;
 
-  this                                  = calloc(1, sizeof(demux_iff_t));
+  this = calloc(1, sizeof(demux_iff_t));
+  if (!this)
+    return NULL;
+
   this->stream                          = stream;
   this->input                           = input;
 
@@ -1217,26 +1228,28 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-void *demux_iff_init_class (xine_t *xine, void *data) {
-  demux_iff_class_t     *this;
+void *demux_iff_init_class (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_iff_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin         = open_plugin;
-  this->demux_class.description         = N_("IFF demux plugin");
-  this->demux_class.identifier          = "IFF";
-  this->demux_class.mimetypes           =
-    "audio/x-8svx: 8svx: IFF-8SVX Audio;"
-    "audio/8svx: 8svx: IFF-8SVX Audio;"
-    "audio/x-16sv: 16sv: IFF-16SV Audio;"
-    "audio/168sv: 16sv: IFF-16SV Audio;"
-    "image/x-ilbm: ilbm: IFF-ILBM Picture;"
-    "image/ilbm: ilbm: IFF-ILBM Picture;"
-    "video/x-anim: anim: IFF-ANIM Video;"
-    "video/anim: anim: IFF-ANIM Video;";
-  this->demux_class.extensions          = "iff svx 8svx 16sv ilbm ham ham6 ham8 anim anim3 anim5 anim7 anim8";
-  this->demux_class.dispose             = default_demux_class_dispose;
+  static const demux_class_t demux_iff_class = {
+    .open_plugin         = open_plugin,
+    .description         = N_("IFF demux plugin"),
+    .identifier          = "IFF",
+    .mimetypes           =
+      "audio/x-8svx: 8svx: IFF-8SVX Audio;"
+      "audio/8svx: 8svx: IFF-8SVX Audio;"
+      "audio/x-16sv: 16sv: IFF-16SV Audio;"
+      "audio/168sv: 16sv: IFF-16SV Audio;"
+      "image/x-ilbm: ilbm: IFF-ILBM Picture;"
+      "image/ilbm: ilbm: IFF-ILBM Picture;"
+      "video/x-anim: anim: IFF-ANIM Video;"
+      "video/anim: anim: IFF-ANIM Video;",
+    .extensions          = "iff svx 8svx 16sv ilbm ham ham6 ham8 anim anim3 anim5 anim7 anim8",
+    .dispose             = NULL,
+  };
 
-  return this;
+  return (void *)&demux_iff_class;
 }
 
