@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -173,10 +173,6 @@ typedef struct {
   int                  default_video_channel;
 } demux_str_t;
 
-typedef struct {
-  demux_class_t     demux_class;
-} demux_str_class_t;
-
 
 /* returns 1 if the STR file was opened successfully, 0 otherwise */
 static int open_str_file(demux_str_t *this) {
@@ -185,7 +181,8 @@ static int open_str_file(demux_str_t *this) {
 
   memset(this->channel_type, 0, sizeof(this->channel_type));
 
-  this->input->seek(this->input, 0, SEEK_SET);
+  if (this->input->seek(this->input, 0, SEEK_SET) != 0)
+    return 0;
   if (this->input->read(this->input, check_bytes, STR_CHECK_BYTES) !=
       STR_CHECK_BYTES) {
     lprintf("read error\n");
@@ -277,8 +274,11 @@ static int open_str_file(demux_str_t *this) {
 
     /* seek to the next sector and read in the header */
     local_offset = 0;
-    this->input->seek(this->input, this->data_start +
-		      ((sector+1) * CD_RAW_SECTOR_SIZE), SEEK_SET);
+    if (this->input->seek(this->input, this->data_start +
+                          ((sector+1) * CD_RAW_SECTOR_SIZE), SEEK_SET) < 0) {
+      lprintf("sector %d seek error\n", sector);
+      return 0;
+    }
     if (this->input->read(this->input, check_bytes, 0x30) != 0x30) {
       lprintf("sector %d read error\n", sector);
       return 0;
@@ -498,6 +498,9 @@ static int demux_str_seek (demux_plugin_t *this_gen, off_t start_pos, int start_
   start_pos = (off_t) ( (double) start_pos / 65535 *
               this->data_size );
 
+  (void)start_time;
+  (void)playing;
+
   _x_demux_flush_engine (this->stream);
 
   /* round to ensure we start on a sector */
@@ -529,11 +532,15 @@ static int demux_str_get_stream_length (demux_plugin_t *this_gen) {
 }
 
 static uint32_t demux_str_get_capabilities(demux_plugin_t *this_gen) {
+  (void)this_gen;
   return DEMUX_CAP_NOCAP;
 }
 
 static int demux_str_get_optional_data(demux_plugin_t *this_gen,
                                            void *data, int data_type) {
+  (void)this_gen;
+  (void)data;
+  (void)data_type;
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
@@ -547,7 +554,10 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
     return NULL;
   }
 
-  this         = calloc(1, sizeof(demux_str_t));
+  this = calloc(1, sizeof(demux_str_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->input  = input;
 
@@ -583,17 +593,19 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-void *demux_str_init_plugin (xine_t *xine, void *data) {
-  demux_str_class_t     *this;
+void *demux_str_init_plugin (xine_t *xine, const void *data) {
 
-  this = calloc(1, sizeof(demux_str_class_t));
+  (void)xine;
+  (void)data;
 
-  this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.description     = N_("Sony Playstation STR file demux plugin");
-  this->demux_class.identifier      = "PSX STR";
-  this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = "str iki ik2 dps dat xa xa1 xa2 xas xap";
-  this->demux_class.dispose         = default_demux_class_dispose;
+  static const demux_class_t demux_str_class = {
+    .open_plugin     = open_plugin,
+    .description     = N_("Sony Playstation STR file demux plugin"),
+    .identifier      = "PSX STR",
+    .mimetypes       = NULL,
+    .extensions      = "str iki ik2 dps dat xa xa1 xa2 xas xap",
+    .dispose         = NULL,
+  };
 
-  return this;
+  return (void *)&demux_str_class;
 }
