@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2021 the xine project
  * March 2003 - Miguel Freitas
  * This plugin was sponsored by 1Control
  *
@@ -375,6 +375,8 @@ static pvrscr_t *XINE_MALLOC pvrscr_init (void) {
   pvrscr_t *this;
 
   this = calloc(1, sizeof(pvrscr_t));
+  if (!this)
+    return NULL;
 
   this->scr.interface_version = 3;
   this->scr.get_priority      = pvrscr_get_priority;
@@ -932,8 +934,7 @@ static void pvr_finish_recording (pvr_input_plugin_t *this) {
 
       data.mode = 0;
       data.id = show->id;
-      strncpy(data.name, show->base_name, sizeof(data.name));
-      data.name[sizeof(data.name) - 1] = '\0';
+      strlcpy(data.name, show->base_name, sizeof(data.name));
 
       xine_event_send(this->stream, &event);
     } else {
@@ -1006,7 +1007,8 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
         this->input = v4l2_data->input;
 
         /* as of ivtv 0.10.6: must close and reopen to set input */
-        close(this->dev_fd);
+        if (this->dev_fd >= 0)
+          close(this->dev_fd);
         this->dev_fd = xine_open_cloexec(this->devname, O_RDWR);
         if (this->dev_fd < 0) {
           xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
@@ -1156,7 +1158,8 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
        pthread_mutex_lock(&this->dev_lock);
 
        /* how lame. we must close and reopen to change bitrate. */
-       close(this->dev_fd);
+       if (this->dev_fd >= 0)
+         close(this->dev_fd);
        this->dev_fd = xine_open_cloexec(this->devname, O_RDWR);
        if (this->dev_fd == -1) {
          pthread_mutex_unlock(&this->dev_lock);
@@ -1448,12 +1451,16 @@ static int pvr_plugin_open (input_plugin_t *this_gen ) {
   _x_assert(this->scr == NULL);
   time = this->stream->xine->clock->get_current_time(this->stream->xine->clock);
   this->scr = pvrscr_init();
+  if (!this->scr)
+    return 0;
   this->scr->scr.start(&this->scr->scr, time);
   this->stream->xine->clock->register_scr(this->stream->xine->clock, &this->scr->scr);
   this->scr_tunning = 0;
 
   _x_assert(this->event_queue == NULL);
   this->event_queue = xine_event_new_queue (this->stream);
+  if (!this->event_queue)
+    return 0;
 
   /* enable resample method */
   this->stream->xine->config->update_num(this->stream->xine->config,"audio.synchronization.av_sync_method",1);
@@ -1479,21 +1486,22 @@ static input_plugin_t *pvr_class_get_instance (input_class_t *cls_gen, xine_stre
 				    const char *data) {
 
   pvr_input_plugin_t  *this;
-  char                *mrl;
   char                *aux;
 
   if (strncasecmp (data, "pvr:/", 5))
     return NULL;
 
-  mrl = strdup(data);
-  aux = &mrl[5];
-
   this = calloc(1, sizeof (pvr_input_plugin_t));
+  if (!this)
+    return NULL;
+
   this->stream       = stream;
   this->dev_fd       = -1;
-  this->mrl          = mrl;
+  this->mrl          = strdup(data);
   this->max_page_age = 3;
 
+  aux = &this->mrl[5];
+
   /* decode configuration options from mrl */
   if( strlen(aux) ) {
     this->tmp_prefix = strdup(aux);
